# -*- coding: utf-8 -*-

"""
Messaging classes.

@author: Tommy Azzino [tommy.azzino@gmail.com]
"""

import socket
from threading import Thread

class MessagingServer(Thread):
    def __init__(self, ip, port, msg_q=None, debug=False):
        super().__init__()
        self.ip = ip
        self.port = port
        self.msg_q = msg_q
        self.debug = debug
        self.message_sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM, 0)
        self.message_sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
        print("Binding socket to IP: {}, PORT: {}".format(self.ip, self.port))
        self.message_sock.bind((self.ip, self.port))

    def run(self):
        self.message_sock.listen()
        conn, addr = self.message_sock.accept()
        with conn:
            while True:
                message = self.msg_q.get()
                if self.debug:
                    print("Sending message: ", message)
                conn.sendall(message.encode("utf-8"))

class MessagingClient(Thread):
    def __init__(self, ip, port, msg_q=None, debug=False):
        super().__init__()
        self.ip = ip
        self.port = port
        self.msg_q = msg_q
        self.debug = debug
        self.message_sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM, 0)
        print("Connecting socket to IP: {}, PORT: {}".format(self.ip, self.port))
        self.message_sock.connect((self.ip, self.port))

    def run(self):
        while True:
            message = self.message_sock.recv(1024)
            if self.debug:
                print("Receveid message: ", message)
            self.msg_q.put(message.decode("utf-8"))



# -*- coding: utf-8 -*-

"""
Video streaming utils

@author: Tommy Azzino [tommy.azzino@gmail.com]
"""

import pyric.pyw as pyw
from subprocess import Popen, PIPE


def get_wireless_iface():
    target_if_idx = None
    wifs = pyw.winterfaces()
    for i, wif in enumerate(wifs):
        try:
            w = pyw.getcard(wif)
            if pyw.isup(w):
                target_if_idx = i
        except:
            target_if_idx = i
            continue
    if target_if_idx is not None:
        return wifs[target_if_idx]
    return target_if_idx

def get_eth_iface():
    target_if = None
    n = 0
    while n < 10:
        iface = "eth"+str(n)
        if pyw.isinterface(iface):
            try:
                e = pyw.getcard(iface)
                if pyw.isconnected(e):
                    target_if = iface 
                    break
            except:
                target_if = iface
                break
        n+=1

    if target_if is None:
        interfaces = pyw.interfaces()
        for interface in interfaces:
            if "en" in interface:
                target_if = interface

    return target_if

def get_cam():
    cmd = ["/usr/bin/v4l2-ctl", "--list-devices"]
    out, err = Popen(cmd, stdout=PIPE, stderr=PIPE).communicate()
    out, err = out.strip(), err.strip()
    for l in [i.split("\n\t") for i in out.decode().split("\n\n")]:
        if "Arducam" in l[0]:
            return l[1]
    return None

